#include "AlphaInterface.h"

#ifdef __GNUC__
//Unix Functions
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>

int port;

uint32_t CreateInterface(char port_name[])
{
	port = open(port_name, O_RDWR | O_NOCTTY);
	if (port < 0)
	{
		return -1;
	}
	return 0;
}

uint32_t SetInterfaceBaud(uint32_t baudRate)
{
	uint32_t linuxBaudRate;
	switch(baudRate)
	{
	case 0:
		linuxBaudRate = B0;
		break;
	case 50:
		linuxBaudRate = B50;
		break;
	case 75:
		linuxBaudRate = B75;
		break;
	case 110:
		linuxBaudRate = B110;
		break;
	case 134:
		linuxBaudRate = B134;
		break;
	case 150:
		linuxBaudRate = B150;
		break;
	case 200:
		linuxBaudRate = B200;
		break;
	case 300:
		linuxBaudRate = B300;
		break;
	case 600:
		linuxBaudRate = B600;
		break;
	case 1200:
		linuxBaudRate = B1200;
		break;
	case 2400:
		linuxBaudRate = B2400;
		break;
	case 4800:
		linuxBaudRate = B4800;
		break;
	case 9600:
		linuxBaudRate = B9600;
		break;
	case 19200:
		linuxBaudRate = B19200;
		break;
	case 38400:
		linuxBaudRate = B38400;
		break;
	default:
		linuxBaudRate = B19200;
		break;
	}
	return linuxBaudRate;
}
uint32_t SetInterfaceParameters(uint32_t baudRate)
{
	struct termios term;
	term.c_cflag = SetInterfaceBaud(baudRate);
	term.c_cflag = term.c_cflag | CS8 | CREAD;
	term.c_iflag = 0;
	term.c_oflag = 0;
	term.c_lflag = 0;
	tcflush(port, TCIFLUSH);
	tcsetattr(port, TCSANOW, &term);
	return 0;
}

uint32_t SetInterfaceTimeouts(uint32_t readIntervalTimeout)
{
	struct termios term;
	tcgetattr(port, &term);
	term.c_cc[VTIME] = readIntervalTimeout;
	tcsetattr(port, TCSANOW, &term);
	return 0;
}

uint32_t DestroyInterface(void)
{
	close(port);
}

uint32_t Write(uint8_t data[], uint32_t length)
{
	write(port, data, length);
	return length;
}

uint32_t Read(uint8_t data[], uint32_t length)
{
	int offset = 0;
	int n = 0;
	do 
	{
		n = read(port, data + offset, length);
		offset += n;
	} while (offset < length);
	return offset;	
}

#else
//Windows Functions
#include <windows.h>

HANDLE port;

uint32_t CreateInterface(char port_name[])
{
	port = CreateFile(port_name, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (port == INVALID_HANDLE_VALUE)
	{
		DWORD error_code = GetLastError();
		return error_code;
	}
	return 0;
}

uint32_t SetInterfaceParameters(uint32_t baudRate)
{
	DCB serial_parameters = { 0 };
	serial_parameters.DCBlength = sizeof(serial_parameters);
	if (GetCommState(port, &serial_parameters) == 0)
	{
		uint32_t error_code = GetLastError();
		DestroyInterface();
		return error_code;
	}	
	serial_parameters.BaudRate = baudRate;
	serial_parameters.ByteSize = 8;
	serial_parameters.Parity = 0;
	serial_parameters.StopBits = 0;
	if (SetCommState(port, &serial_parameters) == 0)
	{
		uint32_t error_code = GetLastError();
		DestroyInterface();
		return error_code;
	}
	return 0;
}

uint32_t SetInterfaceTimeouts(uint32_t readIntervalTimeout)
{
	COMMTIMEOUTS serial_timeouts = { 0 };
	serial_timeouts.ReadIntervalTimeout = readIntervalTimeout;
	serial_timeouts.ReadTotalTimeoutConstant = 100;
	serial_timeouts.ReadTotalTimeoutMultiplier = 10;
	serial_timeouts.WriteTotalTimeoutConstant = 100;
	serial_timeouts.WriteTotalTimeoutMultiplier = 10;
	if (SetCommTimeouts(port, &serial_timeouts) == 0)
	{
		uint32_t error_code = GetLastError();
		DestroyInterface();
		return error_code;
	}
	return 0;
}

uint32_t DestroyInterface(void)
{
	if (!CloseHandle(port))
	{
		return GetLastError();
	}
	return 0;
}

uint32_t Write(uint8_t data[], uint32_t length)
{
	uint32_t bytes_written = 0;
	WriteFile(port, data, length, (LPDWORD)&bytes_written, NULL);
	return bytes_written;
}

uint32_t Read(uint8_t data[], uint32_t length)
{
	uint32_t bytes_read = 0;
	ReadFile(port, data, length, (LPDWORD)&bytes_read, NULL);
	return bytes_read;
}

#endif
